/* diagnostics.h */
/* Fichier d'entête gestion d'erreurs et diagnostic */
/* A activer avec #define DIAGNOSTICS ou gcc -DDIAGNOSTICS */
/* CMI -- 2008/09 -- stage C */

#ifndef DIAGNOSTICS_H
#define DIAGNOSTICS_H

#ifdef DIAGNOSTICS

#include <stdio.h>
#include <stdlib.h>
/* Affiche le nom du fichier et la ligne où
   est effectué le diagnostic msg */
#define DIAG(msg) fprintf(stderr,"%s:%d:%s\n",__FILE__,__LINE__,msg)
/* Affiche le nom d'une variable entière et sa valeur */
#define VOIR(entier) printf("%s:%u: %s=%u\n", \
                            __FILE__,__LINE__,#entier,entier)
/* Affiche le nom d'une variable caractère et sa valeur */
#define VOIRC(carac) printf("%s:%u: %s=%c\n", \
                            __FILE__,__LINE__,#carac,carac)
/* Affiche le nom d'une variable chaine de caractères et sa valeur */
#define VOIRS(chaine) printf("%s:%u: %s=%s\n", \
                            __FILE__,__LINE__,#chaine,chaine)
/* Affiche un message d'erreur avant de quitter */
#define ERREXIT(msg) {DIAG(msg);exit(EXIT_FAILURE);}
/* Message d'alerte si cond est vérifiée */
#define ALERTE(cond) { if (cond) DIAG(#cond);} 

#else  
/* Aucun affichage */
#define DIAG(msg)
#define VOIR(entier)
#define VOIRC(carac)
#define VOIRS(chaine)
#define ERREXIT(msg)
#define ALERTE(cond)

#endif /* DIAGNOSTICS */

#endif /* DIAGNOSTICS_H */
