/* libchaine.c */
/* Version partielle */
/* L3 Informatique -- 2008/09 -- stage C */

#include <stdio.h>
#include <stdlib.h>

#include "libchaine.h"

/* On active les diagnostics */
#define DIAGNOSTICS
#include "diagnostics.h"

/* 
   Exercice: réécrire cette bibliothèque en commençant par
   typedef char* Chaine;
*/

/* Taille maximale de (certaines) chaînes */
const int LONGMAX = 64;


/* Renvoie une chaîne de caractère de longueur maximale la valeur  */
/* passée en paramètre. Pour limiter les erreurs, tout est mis à 0 */
/* /!\ A utiliser SYSTEMATIQUEMENT pour éviter l'oubli du + 1.     */
char * newChaine(int longueur)
{
  return (char*) calloc((longueur+1), sizeof(char));
}

/* Cette fonction renvoie la longueur de la chaîne passée en paramètre     */
/* Si la chaine est mal formatée (ie pas de caractère de terminaison '\0') */
/* Une erreur de segmentation se produira .....                            */  
int longueur(char *ch)
{
  int i = 0;

  while (ch[i] != '\0')
    i++;
  return i;
}

/* Cette fonction renvoie l'image inversée de son argument */
/* L'allocation mémoire est réalisée par la fonction.      */
char *
miroir(char * argum)
{
  int j , i = 0;
  char * mugra;

  /* Allocation mémoire */
  i = longueur(argum);
  mugra = newChaine(i);
  
  /* facultatif car utilisation de calloc */
  mugra[i] = '\0'; /* On marque la terminaison de la chaîne */
  VOIRS(argum);
  VOIRS(mugra);
  
  i--; /* On se positionne sur la dernière lettre */
  VOIR(i);
  for( j = 0 ; i >= 0  ; i--, j++){
    VOIR(i);
    VOIR(j);
    mugra[j] = argum[i];
  }
  
  VOIRS(mugra);
  return mugra;
}

/* Renvoie la plus grande sous-chaine commençant par carac   */
/* Cette sous-chaine est stockée dans le même espace mémoire */
char * souschaine(char * chaine, char carac)
{
  int i;
  
  while((chaine[i]!=carac) && (chaine[i]!='\0'))
    i++;
  if (chaine[i]=='\0')
    return NULL;
  return chaine+i; /* Correspond à l'adresse où est stocké chaine[i] */
}



/* Compare deux chaines de caractères pour l'ordre alphabétique */
/* Equivalent à strcmp de string.h                              */
int compare(char* ch1, char* ch2)
{
  int i=0; /* Indice */
  VOIR(i);
  
  while ((ch1[i] == ch2[i]) && (ch1[i] != '\0') && (ch2[i] != '\0')){
    VOIR(i);
    VOIRC(ch1[i]);
    VOIRC(ch2[i]);
    i++;
  }
  
  if ((ch1[i]=='\0') && (ch2[i]=='\0'))
    return 0; /* ch1 == ch2 */
  if (ch1[i] > ch2[i])
    return 1; /* ch1 > ch2 */
  return -1; /* ch1 < ch2 */
}

/* Cette  fonction échange deux chaines de caractères passée par  */
/* référence en fonction de leur comparaison alphabétique         */
/* Renvoie 1 si l'échange est effectué                            */
char echange(char* *pch1, char* *pch2)
{
  char* tmp;

  VOIRS(*pch1);
  VOIRS(*pch2);
  if (compare(*pch1,*pch2) == 1)
    {
      tmp = *pch1;
      *pch1 = *pch2;
      *pch2 = tmp;
      VOIRS(*pch1);
      VOIRS(*pch2);
      return 1;
    }
  return 0;
}


/* Cette fonction va lire une chaine au clavier lettre par lettre */
/* Une chaîne trop longue est tronquée */
/* Il n'y a pas de traitement d'erreur */
char * lire_chaine_au_clavier(void)
{
  int c; /* C'est bien un int !! */
  int i=0;
  char *ch;
  
  ch = newChaine(LONGMAX);
  
  c = getchar();
  
  while( (c != '\n') && ( i < LONGMAX ))
    {
      ch[i] = c;
      i++;
      c = getchar();
    }
  /* facultatif car utilisation de calloc */
  ch[i] = '\0';
  
  return ch;
}


/* Cette fonction permet de lire au clavier une chaîne de caractères
   de taille arbitraire */
char * saisie(void)
{
  int c;  /* caractère saisi  /!\ c'est bien un int /!\  */
  char *ch, *tmp; 
  int max = 16; /* taille initiale */
  int i;   /* nb de caractères saisis */
  
  ch = newChaine(max);
  i=0; /* Initialisation */
 
  do{
    c = getchar();
    if (i > max)
      { /* on double la mémoire allouée */
        max *= 2;
        tmp = newChaine(max);
        copieChaine(tmp,ch); /* On recopie ce qui est déjà saisi */
        free(ch); /* ne pas oublier */
        ch = tmp;
      }
    ch[i]=c; /* on stocke le caractère lu */
    i++; 
  }while(c!='\n');
  ch[i-1] = '\0'; /* ne pas oublier d'écraser '\n' par '\0' */
  
  return ch;
}



/* Recopie la chaine de caractère src dans dst  */
/* dst doit être assez grande pour contenir src */
/* Même comportement que strcpy */
int copieChaine(char * dst, char * src)
{   
  int i=0;
  
  while(src[i] != '\0') {
      dst[i] = src[i];
      i++;
    }


  return i;
}


/* Clonage d'une chaine de caractère */
/* Même comportement que strdup */
char* cloneChaine(char * src)
{   
  char * dst; 
  int longsrc;
  
  longsrc = longueur(src);
  dst=newChaine(longsrc); /* allocation mémoire !! */
  copieChaine(dst,src);

  return dst;
}


